@@ -18,6 +18,7 @@ thunar_vfs_job_response_get_type (void)
 	{ THUNAR_VFS_JOB_RESPONSE_NO, "THUNAR_VFS_JOB_RESPONSE_NO", "no" },
 	{ THUNAR_VFS_JOB_RESPONSE_CANCEL, "THUNAR_VFS_JOB_RESPONSE_CANCEL", "cancel" },
 	{ THUNAR_VFS_JOB_RESPONSE_NO_ALL, "THUNAR_VFS_JOB_RESPONSE_NO_ALL", "no-all" },
+	{ THUNAR_VFS_JOB_RESPONSE_RETRY, "THUNAR_VFS_JOB_RESPONSE_RETRY", "retry" },
 	{ 0, NULL, NULL }
 	};
 	type = g_flags_register_static ("ThunarVfsJobResponse", values);
@@ -198,6 +199,7 @@ thunar_vfs_volume_status_get_type (void)
 	static const GFlagsValue values[] = {
 	{ THUNAR_VFS_VOLUME_STATUS_MOUNTED, "THUNAR_VFS_VOLUME_STATUS_MOUNTED", "mounted" },
 	{ THUNAR_VFS_VOLUME_STATUS_PRESENT, "THUNAR_VFS_VOLUME_STATUS_PRESENT", "present" },
+	{ THUNAR_VFS_VOLUME_STATUS_MOUNTABLE, "THUNAR_VFS_VOLUME_STATUS_MOUNTABLE", "mountable" },
 	{ 0, NULL, NULL }
 	};
 	type = g_flags_register_static ("ThunarVfsVolumeStatus", values);
