@@ -1,4 +1,4 @@
-/* $Id: thunar-vfs-path.c 26421 2007-12-02 13:46:28Z benny $ */
+/* $Id: thunar-vfs-path.c 28986 2008-12-07 17:32:35Z nick $ */
 /*-
  * Copyright (c) 2005-2007 Benedikt Meurer <benny@xfce.org>
  *
@@ -1014,7 +1014,7 @@ thunar_vfs_path_list_to_string (GList *path_list)
   gsize  bufsize = 512;
   gsize  bufpos = 0;
   GList *lp;
-  gint   n;
+  guint  n;
 
   /* allocate initial buffer */
   buffer = g_malloc (bufsize + 1);
@@ -1417,7 +1417,7 @@ _thunar_vfs_path_translate (ThunarVfsPath      *src_path,
   else
     {
       /* cannot perform the translation */
-      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_INVAL, g_strerror (EINVAL));
+      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_INVAL, "%s", g_strerror (EINVAL));
     }
 
   return dst_path;
