@@ -1,4 +1,4 @@
-/* $Id: thunar-icon-renderer.c 26421 2007-12-02 13:46:28Z benny $ */
+/* $Id: thunar-icon-renderer.c 28811 2008-11-15 12:06:38Z jannis $ */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
  *
@@ -504,7 +504,7 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
               emblem_area.height = gdk_pixbuf_get_height (emblem);
 
               /* shrink insane emblems */
-              if (G_UNLIKELY (MAX (emblem_area.width, emblem_area.height) > MIN ((2 * icon_renderer->size) / 3, 36)))
+              if (G_UNLIKELY (MAX (emblem_area.width, emblem_area.height) > (gint) MIN ((2 * icon_renderer->size) / 3, 36)))
                 {
                   /* scale down the emblem */
                   temp = exo_gdk_pixbuf_scale_ratio (emblem, MIN ((2 * icon_renderer->size) / 3, 36));
