@@ -1,4 +1,4 @@
-/* $Id: thunar-path-entry.c 26421 2007-12-02 13:46:28Z benny $ */
+/* $Id: thunar-path-entry.c 29340 2009-01-25 12:12:31Z nick $ */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
  *
@@ -663,7 +663,7 @@ thunar_path_entry_button_release_event (GtkWidget      *widget,
 {
   ThunarPathEntry *path_entry = THUNAR_PATH_ENTRY (widget);
 
-  if (event->window == path_entry->icon_area && event->button == path_entry->drag_button)
+  if (event->window == path_entry->icon_area && event->button == (guint) path_entry->drag_button)
     {
       /* reset the drag button state */
       path_entry->drag_button = 0;
@@ -1144,6 +1144,11 @@ thunar_path_entry_match_func (GtkEntryCompletion *completion,
   /* determine the model from the completion */
   model = gtk_entry_completion_get_model (completion);
 
+  /* leave if the model is null, we do this in thunar_path_entry_changed() to speed
+   * things up, but that causes http://bugzilla.xfce.org/show_bug.cgi?id=4847. */
+  if (G_UNLIKELY (model == NULL))
+    return FALSE;
+
   /* determine the current text (UTF-8 normalized) */
   text_normalized = g_utf8_normalize (gtk_entry_get_text (GTK_ENTRY (user_data)), -1, G_NORMALIZE_ALL);
 
@@ -1293,7 +1298,7 @@ thunar_path_entry_parse (ThunarPathEntry *path_entry,
         }
       else
         {
-          g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_INVAL, g_strerror (EINVAL));
+          g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_INVAL, "%s", g_strerror (EINVAL));
         }
     }
 
